@@ -1,5 +1,6 @@
 from schema import Schema, And, Use, Optional
 from lightwood.constants.lightwood import COLUMN_DATA_TYPES
+from lightwood.mixers import NnMixer
 
 feature_schema = Schema({
     'name': str,
@@ -9,22 +10,13 @@ feature_schema = Schema({
     Optional('depends_on_column'): str,
     Optional('dropout'): float,
     Optional('weights'): dict,
-    Optional('secondary_type'): And(str, Use(str.lower), lambda s: s in COLUMN_DATA_TYPES.get_attributes().values())
-})
-
-mixer_graph_schema = Schema({
-    'name': str,
-    'input': list,
-    Optional('output'): list,
-    'class': object,
-    Optional('attrs'): dict
+    Optional('secondary_type'): And(str, Use(str.lower), lambda s: s in COLUMN_DATA_TYPES.get_attributes().values()),
+    Optional('additional_info'): dict
 })
 
 mixer_schema = Schema({
-    Optional('class'): object,
-    Optional('attrs'): dict,
-    Optional('deterministic', default=True): bool,
-    Optional('selfaware', default=True): bool
+    Optional('class', default=NnMixer): object,
+    Optional('kwargs', default={}): dict
 })
 
 data_source_schema = Schema({
@@ -39,6 +31,6 @@ predictor_config_schema = Schema({
         feature_schema
     ],
     Optional('data_source', default=data_source_schema.validate({})): data_source_schema,
-    Optional('mixer', default=mixer_schema.validate({})): mixer_schema,
+    Optional('mixer', default=mixer_schema.validate({'class': NnMixer, 'kwargs': {}})): mixer_schema,
     Optional('optimizer'): object
 })
