@@ -11,7 +11,7 @@ class FlairEmbeddingEncoder(BaseEncoder):
         super().__init__(is_target)
         self.max_sentence = 768
 
-    def prepare_encoder(self, column_data):
+    def prepare(self, column_data):
         # @TODO Maybe allow for an `aim` parameter and set a simpler embedding if the aim is speed (see img_2_vec as an example)
         self.embedding = TransformerDocumentEmbeddings('roberta-base')
 
