@@ -1,3 +1,4 @@
+'''
 import math
 import unittest
 from lightwood.encoders.time_series import TsFreshTsEncoder
@@ -7,8 +8,9 @@ class TestTsFreshTs(unittest.TestCase):
         data = [' '.join(str(math.sin(i / 100)) for i in range(1, 10)) for j in range(20)]
 
         enc = TsFreshTsEncoder()
-        enc.prepare_encoder(data)
+        enc.prepare(data)
         ret = enc.encode(data)
 
         self.assertTrue(len(ret) == len(data))
         self.assertTrue(len(ret) < 60)
+'''
